@@ -29,6 +29,8 @@
 #include "accessors.h"
 #include "file-item.h"
 #include "inode-item.h"
+#include "dir-item.h"
+#include "raid-stripe-tree.h"
 #include "extent-tree.h"
 
 /*
@@ -548,10 +550,9 @@ static int check_dir_item(struct extent_buffer *leaf,
 
 		/* dir type check */
 		dir_type = btrfs_dir_ftype(leaf, di);
-		if (unlikely(dir_type <= BTRFS_FT_UNKNOWN ||
-			     dir_type >= BTRFS_FT_MAX)) {
+		if (unlikely(dir_type >= BTRFS_FT_MAX)) {
 			dir_item_err(leaf, slot,
-			"invalid dir item type, have %u expect (0, %u)",
+			"invalid dir item type, have %u expect [0, %u)",
 				dir_type, BTRFS_FT_MAX);
 			return -EUCLEAN;
 		}
@@ -614,7 +615,7 @@ static int check_dir_item(struct extent_buffer *leaf,
 		 */
 		if (key->type == BTRFS_DIR_ITEM_KEY ||
 		    key->type == BTRFS_XATTR_ITEM_KEY) {
-			char namebuf[MAX(BTRFS_NAME_LEN, XATTR_NAME_MAX)];
+			char namebuf[max(BTRFS_NAME_LEN, XATTR_NAME_MAX)];
 
 			read_extent_buffer(leaf, namebuf,
 					(unsigned long)(di + 1), name_len);
@@ -1266,19 +1267,6 @@ static void extent_err(const struct extent_buffer *eb, int slot,
 	va_end(args);
 }
 
-static bool is_valid_dref_root(u64 rootid)
-{
-	/*
-	 * The following tree root objectids are allowed to have a data backref:
-	 * - subvolume trees
-	 * - data reloc tree
-	 * - tree root
-	 *   For v1 space cache
-	 */
-	return is_fstree(rootid) || rootid == BTRFS_DATA_RELOC_TREE_OBJECTID ||
-	       rootid == BTRFS_ROOT_TREE_OBJECTID;
-}
-
 static int check_extent_item(struct extent_buffer *leaf,
 			     struct btrfs_key *key, int slot,
 			     struct btrfs_key *prev_key)
@@ -1431,8 +1419,6 @@ static int check_extent_item(struct extent_buffer *leaf,
 		struct btrfs_extent_data_ref *dref;
 		struct btrfs_shared_data_ref *sref;
 		u64 seq;
-		u64 dref_root;
-		u64 dref_objectid;
 		u64 dref_offset;
 		u64 inline_offset;
 		u8 inline_type;
@@ -1450,7 +1436,7 @@ static int check_extent_item(struct extent_buffer *leaf,
 		if (unlikely(ptr + btrfs_extent_inline_ref_size(inline_type) > end)) {
 			extent_err(leaf, slot,
 "inline ref item overflows extent item, ptr %lu iref size %u end %lu",
-				   ptr, btrfs_extent_inline_ref_size(inline_type), end);
+				   ptr, inline_type, end);
 			return -EUCLEAN;
 		}
 
@@ -1476,26 +1462,11 @@ static int check_extent_item(struct extent_buffer *leaf,
 		 */
 		case BTRFS_EXTENT_DATA_REF_KEY:
 			dref = (struct btrfs_extent_data_ref *)(&iref->offset);
-			dref_root = btrfs_extent_data_ref_root(leaf, dref);
-			dref_objectid = btrfs_extent_data_ref_objectid(leaf, dref);
 			dref_offset = btrfs_extent_data_ref_offset(leaf, dref);
 			seq = hash_extent_data_ref(
 					btrfs_extent_data_ref_root(leaf, dref),
 					btrfs_extent_data_ref_objectid(leaf, dref),
 					btrfs_extent_data_ref_offset(leaf, dref));
-			if (unlikely(!is_valid_dref_root(dref_root))) {
-				extent_err(leaf, slot,
-					   "invalid data ref root value %llu",
-					   dref_root);
-				return -EUCLEAN;
-			}
-			if (unlikely(dref_objectid < BTRFS_FIRST_FREE_OBJECTID ||
-				     dref_objectid > BTRFS_LAST_FREE_OBJECTID)) {
-				extent_err(leaf, slot,
-					   "invalid data ref objectid value %llu",
-					   dref_objectid);
-				return -EUCLEAN;
-			}
 			if (unlikely(!IS_ALIGNED(dref_offset,
 						 fs_info->sectorsize))) {
 				extent_err(leaf, slot,
@@ -1503,11 +1474,6 @@ static int check_extent_item(struct extent_buffer *leaf,
 					   dref_offset, fs_info->sectorsize);
 				return -EUCLEAN;
 			}
-			if (unlikely(btrfs_extent_data_ref_count(leaf, dref) == 0)) {
-				extent_err(leaf, slot,
-			"invalid data ref count, should have non-zero value");
-				return -EUCLEAN;
-			}
 			inline_refs += btrfs_extent_data_ref_count(leaf, dref);
 			break;
 		/* Contains parent bytenr and ref count */
@@ -1520,13 +1486,11 @@ static int check_extent_item(struct extent_buffer *leaf,
 					   inline_offset, fs_info->sectorsize);
 				return -EUCLEAN;
 			}
-			if (unlikely(btrfs_shared_data_ref_count(leaf, sref) == 0)) {
-				extent_err(leaf, slot,
-			"invalid shared data ref count, should have non-zero value");
-				return -EUCLEAN;
-			}
 			inline_refs += btrfs_shared_data_ref_count(leaf, sref);
 			break;
+		case BTRFS_EXTENT_OWNER_REF_KEY:
+			WARN_ON(!btrfs_fs_incompat(fs_info, SIMPLE_QUOTA));
+			break;
 		default:
 			extent_err(leaf, slot, "unknown inline ref type: %u",
 				   inline_type);
@@ -1594,18 +1558,8 @@ static int check_simple_keyed_refs(struct extent_buffer *leaf,
 {
 	u32 expect_item_size = 0;
 
-	if (key->type == BTRFS_SHARED_DATA_REF_KEY) {
-		struct btrfs_shared_data_ref *sref;
-
-		sref = btrfs_item_ptr(leaf, slot, struct btrfs_shared_data_ref);
-		if (unlikely(btrfs_shared_data_ref_count(leaf, sref) == 0)) {
-			extent_err(leaf, slot,
-		"invalid shared data backref count, should have non-zero value");
-			return -EUCLEAN;
-		}
-
+	if (key->type == BTRFS_SHARED_DATA_REF_KEY)
 		expect_item_size = sizeof(struct btrfs_shared_data_ref);
-	}
 
 	if (unlikely(btrfs_item_size(leaf, slot) != expect_item_size)) {
 		generic_err(leaf, slot,
@@ -1651,8 +1605,6 @@ static int check_extent_data_ref(struct extent_buffer *leaf,
 		return -EUCLEAN;
 	}
 	for (; ptr < end; ptr += sizeof(*dref)) {
-		u64 root;
-		u64 objectid;
 		u64 offset;
 
 		/*
@@ -1660,33 +1612,13 @@ static int check_extent_data_ref(struct extent_buffer *leaf,
 		 * overflow from the leaf due to hash collisions.
 		 */
 		dref = (struct btrfs_extent_data_ref *)ptr;
-		root = btrfs_extent_data_ref_root(leaf, dref);
-		objectid = btrfs_extent_data_ref_objectid(leaf, dref);
 		offset = btrfs_extent_data_ref_offset(leaf, dref);
-		if (unlikely(!is_valid_dref_root(root))) {
-			extent_err(leaf, slot,
-				   "invalid extent data backref root value %llu",
-				   root);
-			return -EUCLEAN;
-		}
-		if (unlikely(objectid < BTRFS_FIRST_FREE_OBJECTID ||
-			     objectid > BTRFS_LAST_FREE_OBJECTID)) {
-			extent_err(leaf, slot,
-				   "invalid extent data backref objectid value %llu",
-				   root);
-			return -EUCLEAN;
-		}
 		if (unlikely(!IS_ALIGNED(offset, leaf->fs_info->sectorsize))) {
 			extent_err(leaf, slot,
 	"invalid extent data backref offset, have %llu expect aligned to %u",
 				   offset, leaf->fs_info->sectorsize);
 			return -EUCLEAN;
 		}
-		if (unlikely(btrfs_extent_data_ref_count(leaf, dref) == 0)) {
-			extent_err(leaf, slot,
-	"invalid extent data backref count, should have non-zero value");
-			return -EUCLEAN;
-		}
 	}
 	return 0;
 }
@@ -1717,10 +1649,10 @@ static int check_inode_ref(struct extent_buffer *leaf,
 	while (ptr < end) {
 		u16 namelen;
 
-		if (unlikely(ptr + sizeof(*iref) > end)) {
+		if (unlikely(ptr + sizeof(iref) > end)) {
 			inode_ref_err(leaf, slot,
 			"inode ref overflow, ptr %lu end %lu inode_ref_size %zu",
-				ptr, end, sizeof(*iref));
+				ptr, end, sizeof(iref));
 			return -EUCLEAN;
 		}
 
@@ -1743,69 +1675,41 @@ static int check_inode_ref(struct extent_buffer *leaf,
 	return 0;
 }
 
-static int check_dev_extent_item(const struct extent_buffer *leaf,
-				 const struct btrfs_key *key,
-				 int slot,
-				 struct btrfs_key *prev_key)
+static int check_raid_stripe_extent(const struct extent_buffer *leaf,
+				    const struct btrfs_key *key, int slot)
 {
-	struct btrfs_dev_extent *de;
-	const u32 sectorsize = leaf->fs_info->sectorsize;
+	struct btrfs_stripe_extent *stripe_extent =
+		btrfs_item_ptr(leaf, slot, struct btrfs_stripe_extent);
 
-	de = btrfs_item_ptr(leaf, slot, struct btrfs_dev_extent);
-	/* Basic fixed member checks. */
-	if (unlikely(btrfs_dev_extent_chunk_tree(leaf, de) !=
-		     BTRFS_CHUNK_TREE_OBJECTID)) {
+	if (unlikely(!IS_ALIGNED(key->objectid, leaf->fs_info->sectorsize))) {
 		generic_err(leaf, slot,
-			    "invalid dev extent chunk tree id, has %llu expect %llu",
-			    btrfs_dev_extent_chunk_tree(leaf, de),
-			    BTRFS_CHUNK_TREE_OBJECTID);
+"invalid key objectid for raid stripe extent, have %llu expect aligned to %u",
+			    key->objectid, leaf->fs_info->sectorsize);
 		return -EUCLEAN;
 	}
-	if (unlikely(btrfs_dev_extent_chunk_objectid(leaf, de) !=
-		     BTRFS_FIRST_CHUNK_TREE_OBJECTID)) {
-		generic_err(leaf, slot,
-			    "invalid dev extent chunk objectid, has %llu expect %llu",
-			    btrfs_dev_extent_chunk_objectid(leaf, de),
-			    BTRFS_FIRST_CHUNK_TREE_OBJECTID);
-		return -EUCLEAN;
-	}
-	/* Alignment check. */
-	if (unlikely(!IS_ALIGNED(key->offset, sectorsize))) {
-		generic_err(leaf, slot,
-			    "invalid dev extent key.offset, has %llu not aligned to %u",
-			    key->offset, sectorsize);
-		return -EUCLEAN;
-	}
-	if (unlikely(!IS_ALIGNED(btrfs_dev_extent_chunk_offset(leaf, de),
-				 sectorsize))) {
-		generic_err(leaf, slot,
-			    "invalid dev extent chunk offset, has %llu not aligned to %u",
-			    btrfs_dev_extent_chunk_objectid(leaf, de),
-			    sectorsize);
-		return -EUCLEAN;
-	}
-	if (unlikely(!IS_ALIGNED(btrfs_dev_extent_length(leaf, de),
-				 sectorsize))) {
-		generic_err(leaf, slot,
-			    "invalid dev extent length, has %llu not aligned to %u",
-			    btrfs_dev_extent_length(leaf, de), sectorsize);
-		return -EUCLEAN;
-	}
-	/* Overlap check with previous dev extent. */
-	if (slot && prev_key->objectid == key->objectid &&
-	    prev_key->type == key->type) {
-		struct btrfs_dev_extent *prev_de;
-		u64 prev_len;
 
-		prev_de = btrfs_item_ptr(leaf, slot - 1, struct btrfs_dev_extent);
-		prev_len = btrfs_dev_extent_length(leaf, prev_de);
-		if (unlikely(prev_key->offset + prev_len > key->offset)) {
-			generic_err(leaf, slot,
-		"dev extent overlap, prev offset %llu len %llu current offset %llu",
-				    prev_key->objectid, prev_len, key->offset);
-			return -EUCLEAN;
-		}
+	if (unlikely(!btrfs_fs_incompat(leaf->fs_info, RAID_STRIPE_TREE))) {
+		generic_err(leaf, slot,
+	"RAID_STRIPE_EXTENT present but RAID_STRIPE_TREE incompat bit unset");
+		return -EUCLEAN;
 	}
+
+	switch (btrfs_stripe_extent_encoding(leaf, stripe_extent)) {
+	case BTRFS_STRIPE_RAID0:
+	case BTRFS_STRIPE_RAID1:
+	case BTRFS_STRIPE_DUP:
+	case BTRFS_STRIPE_RAID10:
+	case BTRFS_STRIPE_RAID5:
+	case BTRFS_STRIPE_RAID6:
+	case BTRFS_STRIPE_RAID1C3:
+	case BTRFS_STRIPE_RAID1C4:
+		break;
+	default:
+		generic_err(leaf, slot, "invalid raid stripe encoding %u",
+			    btrfs_stripe_extent_encoding(leaf, stripe_extent));
+		return -EUCLEAN;
+	}
+
 	return 0;
 }
 
@@ -1845,9 +1749,6 @@ static enum btrfs_tree_block_status check_leaf_item(struct extent_buffer *leaf,
 	case BTRFS_DEV_ITEM_KEY:
 		ret = check_dev_item(leaf, key, slot);
 		break;
-	case BTRFS_DEV_EXTENT_KEY:
-		ret = check_dev_extent_item(leaf, key, slot, prev_key);
-		break;
 	case BTRFS_INODE_ITEM_KEY:
 		ret = check_inode_item(leaf, key, slot);
 		break;
@@ -1866,6 +1767,9 @@ static enum btrfs_tree_block_status check_leaf_item(struct extent_buffer *leaf,
 	case BTRFS_EXTENT_DATA_REF_KEY:
 		ret = check_extent_data_ref(leaf, key, slot);
 		break;
+	case BTRFS_RAID_STRIPE_KEY:
+		ret = check_raid_stripe_extent(leaf, key, slot);
+		break;
 	}
 
 	if (ret)
@@ -1889,11 +1793,6 @@ enum btrfs_tree_block_status __btrfs_check_leaf(struct extent_buffer *leaf)
 		return BTRFS_TREE_BLOCK_INVALID_LEVEL;
 	}
 
-	if (unlikely(!btrfs_header_flag(leaf, BTRFS_HEADER_FLAG_WRITTEN))) {
-		generic_err(leaf, 0, "invalid flag for leaf, WRITTEN not set");
-		return BTRFS_TREE_BLOCK_WRITTEN_NOT_SET;
-	}
-
 	/*
 	 * Extent buffers from a relocation tree have a owner field that
 	 * corresponds to the subvolume tree they are based on. So just from an
@@ -1955,7 +1854,6 @@ enum btrfs_tree_block_status __btrfs_check_leaf(struct extent_buffer *leaf)
 	for (slot = 0; slot < nritems; slot++) {
 		u32 item_end_expected;
 		u64 item_data_end;
-		enum btrfs_tree_block_status ret;
 
 		btrfs_item_key_to_cpu(leaf, &key, slot);
 
@@ -2011,10 +1909,21 @@ enum btrfs_tree_block_status __btrfs_check_leaf(struct extent_buffer *leaf)
 			return BTRFS_TREE_BLOCK_INVALID_OFFSETS;
 		}
 
-		/* Check if the item size and content meet other criteria. */
-		ret = check_leaf_item(leaf, &key, slot, &prev_key);
-		if (unlikely(ret != BTRFS_TREE_BLOCK_CLEAN))
-			return ret;
+		/*
+		 * We only want to do this if WRITTEN is set, otherwise the leaf
+		 * may be in some intermediate state and won't appear valid.
+		 */
+		if (btrfs_header_flag(leaf, BTRFS_HEADER_FLAG_WRITTEN)) {
+			enum btrfs_tree_block_status ret;
+
+			/*
+			 * Check if the item size and content meet other
+			 * criteria
+			 */
+			ret = check_leaf_item(leaf, &key, slot, &prev_key);
+			if (unlikely(ret != BTRFS_TREE_BLOCK_CLEAN))
+				return ret;
+		}
 
 		prev_key.objectid = key.objectid;
 		prev_key.type = key.type;
@@ -2044,11 +1953,6 @@ enum btrfs_tree_block_status __btrfs_check_node(struct extent_buffer *node)
 	int level = btrfs_header_level(node);
 	u64 bytenr;
 
-	if (unlikely(!btrfs_header_flag(node, BTRFS_HEADER_FLAG_WRITTEN))) {
-		generic_err(node, 0, "invalid flag for node, WRITTEN not set");
-		return BTRFS_TREE_BLOCK_WRITTEN_NOT_SET;
-	}
-
 	if (unlikely(level <= 0 || level >= BTRFS_MAX_LEVEL)) {
 		generic_err(node, 0,
 			"invalid level for node, have %d expect [1, %d]",
@@ -2186,7 +2090,7 @@ int btrfs_verify_level_key(struct extent_buffer *eb, int level,
 	 * So we only checks tree blocks which is read from disk, whose
 	 * generation <= fs_info->last_trans_committed.
 	 */
-	if (btrfs_header_generation(eb) > fs_info->last_trans_committed)
+	if (btrfs_header_generation(eb) > btrfs_get_last_trans_committed(fs_info))
 		return 0;
 
 	/* We have @first_key, so this @eb must have at least one item */
