@@ -174,9 +174,7 @@ static int fsl_xcvr_activate_ctl(struct snd_soc_dai *dai, const char *name,
 	struct snd_kcontrol *kctl;
 	bool enabled;
 
-	lockdep_assert_held(&card->snd_card->controls_rwsem);
-
-	kctl = snd_soc_card_get_kcontrol_locked(card, name);
+	kctl = snd_soc_card_get_kcontrol(card, name);
 	if (kctl == NULL)
 		return -ENOENT;
 
@@ -578,14 +576,10 @@ static int fsl_xcvr_startup(struct snd_pcm_substream *substream,
 	xcvr->streams |= BIT(substream->stream);
 
 	if (!xcvr->soc_data->spdif_only) {
-		struct snd_soc_card *card = dai->component->card;
-
 		/* Disable XCVR controls if there is stream started */
-		down_read(&card->snd_card->controls_rwsem);
 		fsl_xcvr_activate_ctl(dai, fsl_xcvr_mode_kctl.name, false);
 		fsl_xcvr_activate_ctl(dai, fsl_xcvr_arc_mode_kctl.name, false);
 		fsl_xcvr_activate_ctl(dai, fsl_xcvr_earc_capds_kctl.name, false);
-		up_read(&card->snd_card->controls_rwsem);
 	}
 
 	return 0;
@@ -604,15 +598,11 @@ static void fsl_xcvr_shutdown(struct snd_pcm_substream *substream,
 	/* Enable XCVR controls if there is no stream started */
 	if (!xcvr->streams) {
 		if (!xcvr->soc_data->spdif_only) {
-			struct snd_soc_card *card = dai->component->card;
-
-			down_read(&card->snd_card->controls_rwsem);
 			fsl_xcvr_activate_ctl(dai, fsl_xcvr_mode_kctl.name, true);
 			fsl_xcvr_activate_ctl(dai, fsl_xcvr_arc_mode_kctl.name,
 						(xcvr->mode == FSL_XCVR_MODE_ARC));
 			fsl_xcvr_activate_ctl(dai, fsl_xcvr_earc_capds_kctl.name,
 						(xcvr->mode == FSL_XCVR_MODE_EARC));
-			up_read(&card->snd_card->controls_rwsem);
 		}
 		ret = regmap_update_bits(xcvr->regmap, FSL_XCVR_EXT_IER0,
 					 FSL_XCVR_IRQ_EARC_ALL, 0);
@@ -1195,28 +1185,8 @@ static irqreturn_t irq0_isr(int irq, void *devid)
 						bitrev32(val);
 				}
 				/* clear CS control register */
-				writel_relaxed(0, reg_ctrl);
+				memset_io(reg_ctrl, 0, sizeof(val));
 			}
-		} else {
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_0,
-				    (u32 *)&xcvr->rx_iec958.status[0]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_1,
-				    (u32 *)&xcvr->rx_iec958.status[4]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_2,
-				    (u32 *)&xcvr->rx_iec958.status[8]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_3,
-				    (u32 *)&xcvr->rx_iec958.status[12]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_4,
-				    (u32 *)&xcvr->rx_iec958.status[16]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_5,
-				    (u32 *)&xcvr->rx_iec958.status[20]);
-			for (i = 0; i < 6; i++) {
-				val = *(u32 *)(xcvr->rx_iec958.status + i * 4);
-				*(u32 *)(xcvr->rx_iec958.status + i * 4) =
-					bitrev32(val);
-			}
-			regmap_set_bits(xcvr->regmap, FSL_XCVR_RX_DPTH_CTRL,
-					FSL_XCVR_RX_DPTH_CTRL_CSA);
 		}
 	}
 	if (isr & FSL_XCVR_IRQ_NEW_UD) {
